@@ -1014,6 +1014,14 @@ class CroniterTest(base.TestCase):
             "0 0 * * *",
             datetime(2019, 1, 14, 0, 1, 0, 0)
         ))
+        self.assertTrue(croniter.match(
+            "0 0 * * * 1",
+            datetime(2023, 5, 25, 0, 0, 1, 0)
+        ))
+        self.assertFalse(croniter.match(
+            "0 0 * * * 1",
+            datetime(2023, 5, 25, 0, 0, 2, 0)
+        ))
         self.assertTrue(croniter.match(
             "31 * * * *",
             datetime(2019, 1, 14, 1, 31, 0, 0)
@@ -1068,8 +1076,8 @@ class CroniterTest(base.TestCase):
             '2020-03-30T02:01:00+02:00',
             '2020-03-29T01:01:00+01:00',
             '2020-03-28T03:01:00+01:00',
-            '2020-03-29T02:01:00+02:00',
-            '2020-03-29T03:01:00+02:00'])
+            '2020-03-29T03:01:00+02:00',
+            '2020-03-30T02:01:00+02:00'])
         #
         nowp = datetime(2020, 3, 28, 1, 58, 55, tzinfo=tz)
         itp = croniter('1 2 * * *', nowp)
@@ -1081,11 +1089,11 @@ class CroniterTest(base.TestCase):
             itp.get_next(datetime).isoformat(),
         ]
         self.assertEqual(retp, [
-            '2020-03-29T02:01:00+02:00',
-            '2020-03-28T02:01:00+01:00',
-            '2020-03-27T02:01:00+01:00',
-            '2020-03-28T02:01:00+01:00',
-            '2020-03-29T03:01:00+02:00'])
+            '2020-03-29T03:01:00+02:00',
+            '2020-03-29T01:01:00+01:00',
+            '2020-03-28T03:01:00+01:00',
+            '2020-03-29T03:01:00+02:00',
+            '2020-03-30T02:01:00+02:00'])
         #
         nowt = datetime(2020, 3, 29, 2, 0, 0, tzinfo=tz)
         itt = croniter('1 2 * * *', nowt)
@@ -1100,9 +1108,22 @@ class CroniterTest(base.TestCase):
             '2020-03-30T02:01:00+02:00',
             '2020-03-29T01:01:00+01:00',
             '2020-03-28T03:01:00+01:00',
-            '2020-03-29T02:01:00+02:00',
-            '2020-03-29T03:01:00+02:00'])
+            '2020-03-29T03:01:00+02:00',
+            '2020-03-30T02:01:00+02:00'])
 
+    def test_dst_iter(self):
+        tz = pytz.timezone('Asia/Hebron')
+        now = datetime(2022, 3, 26, 0, 0, 0, tzinfo=tz)
+        it = croniter('0 0 * * *', now)
+        ret = [
+            it.get_next(datetime).isoformat(),
+            it.get_next(datetime).isoformat(),
+            it.get_next(datetime).isoformat(),
+        ]
+        self.assertEqual(ret, [
+            '2022-03-26T00:00:00+02:00',
+            '2022-03-27T01:00:00+03:00',
+            '2022-03-28T00:00:00+03:00'])
 
     def test_nth_wday_simple(self):
         f = lambda y,m,w: croniter._get_nth_weekday_of_month(y,m,w)
@@ -1460,6 +1481,72 @@ class CroniterTest(base.TestCase):
 
         self.assertEqual(n1, datetime_to_timestamp(base) + 60)
 
+    def test_issue_k34(self):
+        # invalid cron, but should throw appropriate exception
+        self.assertRaises(CroniterBadCronError, croniter, "4 0 L/2 2 0")
+
+    def test_issue_k33(self):
+        y = 2018
+        # At 11:30 PM, between day 1 and 7 of the month, Monday through Friday, only in January
+        ret = []
+        for i in range(10):
+            cron = croniter("30 23 1-7 JAN MON-FRI#1", datetime(y+i, 1, 1), ret_type=datetime)
+            for j in range(7):
+                d = cron.get_next()
+                if d.year == y + i:
+                    ret.append(d)
+        rets = [datetime(2018, 1, 1, 23, 30),
+                datetime(2018, 1, 2, 23, 30),
+                datetime(2018, 1, 3, 23, 30),
+                datetime(2018, 1, 4, 23, 30),
+                datetime(2018, 1, 5, 23, 30),
+                datetime(2019, 1, 1, 23, 30),
+                datetime(2019, 1, 2, 23, 30),
+                datetime(2019, 1, 3, 23, 30),
+                datetime(2019, 1, 4, 23, 30),
+                datetime(2019, 1, 7, 23, 30),
+                datetime(2020, 1, 1, 23, 30),
+                datetime(2020, 1, 2, 23, 30),
+                datetime(2020, 1, 3, 23, 30),
+                datetime(2020, 1, 6, 23, 30),
+                datetime(2020, 1, 7, 23, 30),
+                datetime(2021, 1, 1, 23, 30),
+                datetime(2021, 1, 4, 23, 30),
+                datetime(2021, 1, 5, 23, 30),
+                datetime(2021, 1, 6, 23, 30),
+                datetime(2021, 1, 7, 23, 30),
+                datetime(2022, 1, 3, 23, 30),
+                datetime(2022, 1, 4, 23, 30),
+                datetime(2022, 1, 5, 23, 30),
+                datetime(2022, 1, 6, 23, 30),
+                datetime(2022, 1, 7, 23, 30),
+                datetime(2023, 1, 2, 23, 30),
+                datetime(2023, 1, 3, 23, 30),
+                datetime(2023, 1, 4, 23, 30),
+                datetime(2023, 1, 5, 23, 30),
+                datetime(2023, 1, 6, 23, 30),
+                datetime(2024, 1, 1, 23, 30),
+                datetime(2024, 1, 2, 23, 30),
+                datetime(2024, 1, 3, 23, 30),
+                datetime(2024, 1, 4, 23, 30),
+                datetime(2024, 1, 5, 23, 30),
+                datetime(2025, 1, 1, 23, 30),
+                datetime(2025, 1, 2, 23, 30),
+                datetime(2025, 1, 3, 23, 30),
+                datetime(2025, 1, 6, 23, 30),
+                datetime(2025, 1, 7, 23, 30),
+                datetime(2026, 1, 1, 23, 30),
+                datetime(2026, 1, 2, 23, 30),
+                datetime(2026, 1, 5, 23, 30),
+                datetime(2026, 1, 6, 23, 30),
+                datetime(2026, 1, 7, 23, 30),
+                datetime(2027, 1, 1, 23, 30),
+                datetime(2027, 1, 4, 23, 30),
+                datetime(2027, 1, 5, 23, 30),
+                datetime(2027, 1, 6, 23, 30),
+                datetime(2027, 1, 7, 23, 30)]
+        self.assertEqual(ret, rets)
+        croniter.expand("30 6 1-7 MAY MON#1")
 
 if __name__ == '__main__':
     unittest.main()
